//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __InjGen__
#define __InjGen__

#include "TMath.h"
#include "TRandom3.h"
#include "Segments.h"
#include "InjTre.h"

using namespace std;

/**
 * Generate a set of injections.
 * \author    Florent Robinet
 */
class InjGen : public InjTre{
  
 public:
  
  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the InjGen class.
   * Injections are performed over a set of Segments.
   * The injection type must be provided:
   * - "Gauss" for Gaussian injections
   * - "SineGauss"  for sine-Gaussian injections
   *
   * @param aInSegments input segments
   * @param aInjType injection type
   * @param aVerbose verbosity level
   */
  InjGen(Segments *aInSegments, const string aInjType, const int aVerbose=0);

  /**
   * Destructor of the InjTre class.
   */
  virtual ~InjGen();
  /**
     @}
  */


  /**
   * Generates a set of injection parameters.
   * Returns the number of injections.
   */
  int GenerateInjections(void);

  /**
   * Sets time space between 2 consecutive injections.
   * @param aSpacing time space [s]
   */
  inline void SetInjectionSpacing(const double aSpacing){ InjSpacing=aSpacing; };
  
  /**
   * Sets the injection amplitude distribution.
   * Amplitudes can either be logarithmically distributed ("LOG") or uniformly distributed ("UNIFORM").
   * @param aDistribution distribution type
   */
  bool SetAmplitudeDistribution(const string aDistribution="LOG");

  /**
   * Sets the injection amplitude range.
   * @param aMin minimum amplitude
   * @param aMax maximum amplitude
   */
  bool SetAmplitudeRange(const double aMin=1e-22, const double aMax=5e-19);

  /**
   * Sets the injection sigma distribution.
   * Sigmas can either be logarithmically distributed ("LOG") or uniformly distributed ("UNIFORM").
   * @param aDistribution distribution type
   */
  bool SetSigmaDistribution(const string aDistribution="UNIFORM");

  /**
   * Sets the injection sigma range.
   * @param aMin minimum sigma [s]
   * @param aMax maximum sigma [s]
   */
  bool SetSigmaRange(const double aMin=0.01, const double aMax=1.0);

  /**
   * Sets the injection frequency distribution.
   * Frequencies can either be logarithmically distributed ("LOG") or uniformly distributed ("UNIFORM").
   * @param aDistribution distribution type
   */
  bool SetFrequencyDistribution(const string aDistribution="LOG");

  /**
   * Sets the injection frequency range.
   * @param aMin minimum frequency [Hz]
   * @param aMax maximum frequency [Hz]
   */
  bool SetFrequencyRange(const double aMin=30.0, const double aMax=2048.0);

  /**
   * Sets the injection eccentricity distribution.
   * Eccentricities can either be logarithmically distributed ("LOG") or uniformly distributed ("UNIFORM").
   * @param aDistribution distribution type
   */
  bool SetEccentricityDistribution(const string aDistribution="UNIFORM");

  /**
   * Sets the injection eccentricity range.
   * @param aMin minimum eccentricity
   * @param aMax maximum eccentricity
   */
  bool SetEccentricityRange(const double aMin=0.0, const double aMax=1.0);

  /**
   * Sets the injection polarization distribution.
   * Polarizations can either be logarithmically distributed ("LOG") or uniformly distributed ("UNIFORM").
   * @param aDistribution distribution type
   */
  bool SetPolarizationDistribution(const string aDistribution="UNIFORM");

  /**
   * Sets the injection polarization range.
   * @param aMin minimum polarization [rad]
   * @param aMax maximum polarization [rad]
   */
  bool SetPolarizationRange(const double aMin=0.0, const double aMax=2.0*TMath::Pi());

 private:
  
  Segments *InSegments;        ///< input segments - DO NOT DELETE
  double InjSpacing;           ///< injection spacing [s]

  TRandom3 *rand;              ///< random generator

  
  // AMPLITUDE
  double GetAmplitude(void);   ///< generate amplitude
  double ampMin,               ///< amplitude min
    ampMax;                    ///< amplitude max
  int ampDistribution;         ///< 0=UNIFORM, 1=LOG/10

  // SIGMA
  double GetSigma(void);       ///< generate sigma
  double sigmaMin,             ///< sigma min
    sigmaMax;                  ///< sigma max
  int sigmaDistribution;       ///< 0=UNIFORM, 1=LOG/10
  
  // F0
  double GetFrequency(void);   ///< generate frequency
  double f0Min,                ///< frequency min
    f0Max;                     ///< frequency max
  int f0Distribution;          ///< 0=UNIFORM, 1=LOG/10

  // ECCENTRICITY
  double GetEccentricity(void);///< generate eccentricity
  double eccMin,               ///< eccentricity min
    eccMax;                    ///< eccentricity max
  int eccDistribution;         ///< 0=UNIFORM, 1=LOG/10

  // POLARIZATION
  double GetPolarization(void);///< generate polarization
  double psiMin,               ///< psi min
    psiMax;                    ///< psi max
  int psiDistribution;         ///< 0=UNIFORM, 1=LOG/10

  ClassDef(InjGen,0)
};

#endif


