//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __InjTre__
#define __InjTre__


#include "CUtils.h"
#include "TFile.h"
#include "TTree.h"

using namespace std;

/**
 * Manage injection TTree.
 * \author    Florent Robinet
 */
class InjTre {
  
 public:
  
  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the InjTre class.
   * The injection type must be provided:
   * - "Gauss" for Gaussian injections
   * - "SineGauss"  for sine-Gaussian injections
   *
   * @param aInjType injection type
   * @param aVerbose verbosity level
   */
  InjTre(const string aInjType, const int aVerbose=0);
  
  /**
   * Destructor of the InjTre class.
   */
  virtual ~InjTre();
  /**
     @}
  */

  /**
   * Adds a Gauss-type injection.
   * Returns the current number of injections in TTree. Returns -1 if this function fails.
   * @param aRa injection right ascension [rad]
   * @param aDec injection declination [rad]
   * @param aTime injection GPS time
   * @param aEccentricity injection eccentricity
   * @param aPolarization injection polarization angle [rad]
   * @param aAmplitude injection amplitude
   * @param aSigma injection sigma [s]
   */
  int AddGauss(const double aRa, 
	       const double aDec, 
	       const double aTime, 
	       const double aEccentricity, 
	       const double aPolarization, 
	       const double aAmplitude, 
	       const double aSigma);

  /**
   * Adds a SineGauss-type injection.
   * Returns the current number of injections in TTree. Returns -1 if this function fails.
   * @param aRa injection right ascension [rad]
   * @param aDec injection declination [rad]
   * @param aTime injection GPS time
   * @param aEccentricity injection eccentricity
   * @param aPolarization injection polarization angle [rad]
   * @param aAmplitude injection amplitude
   * @param aSigma injection sigma [s]
   * @param aFrequency injection frequency [Hz]
   */
  int AddSineGauss(const double aRa, 
		   const double aDec, 
		   const double aTime, 
		   const double aEccentricity, 
		   const double aPolarization, 
		   const double aAmplitude, 
		   const double aSigma, 
		   const double aFrequency);

  /**
   * Returns the current number of injections.
   */
  inline int GetNInjections(void){ return InjTree->GetEntries(); };

  /**
   * Returns the injection type.
   */
  inline string GetInjectionType(void){ return (string)InjTree->GetTitle(); };

  /**
   * Resets injection TTree.
   */
  inline void Reset(void){ return InjTree->Reset(); };

  /**
   * Writes injections on disk.
   * @param aRootFileName output file name
   */  
  bool Write(const string aRootFileName="./myinjections.root");
  

 protected:
  
  // INPUT
  int Verbose;               ///< verbosity level
  
 private:  

  TTree *InjTree;            ///< injection tree
  double inj_ra,             ///< injection right ascension
    inj_dec,                 ///< injection declination
    inj_psi,                 ///< injection polarization angle
    inj_time,                ///< injection time
    inj_amp,                 ///< injection amplitude
    inj_f0,                  ///< injection frequency
    inj_sigma,               ///< injection sigma
    inj_ecc;                 ///< injection eccentricity

  ClassDef(InjTre,0)
};

#endif


